﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirBomber
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetAirplanesGeneric<T>
        where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly T[] _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Length;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetAirplanesGeneric(int count)
        {
            _places = new T[count];
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="airplane">Добавляемый самолет</param>
        /// <returns></returns>
        public bool Insert(T airplane)
        {
            return Insert(airplane, 0);
        }

        private bool isCorrectPosition(int position)
        {
            return 0 <= position && position < Count;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="airplane">Добавляемый самолет</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public bool Insert(T airplane, int position)
        {
            int positionNullElement = position;
            while (Get(positionNullElement) != null)
            {
                positionNullElement++;
            }
            // Если изначальная позиция была некорректной или пустых элементов справа не оказалось возвращаем false
            if (!isCorrectPosition(positionNullElement)) 
            {
                return false;
            }
            while (positionNullElement != position) // Смещение вправо
            {
                _places[positionNullElement] = _places[positionNullElement - 1];
                positionNullElement--;
            }
            _places[position] = airplane;
            return true;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public bool Remove(int position)
        {
            if (!isCorrectPosition(position))
                return false;
            _places[position] = null;
            return true;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Get(int position)
        {
            return isCorrectPosition(position) ? _places[position] : null;
        }
    }
}
